package it.openutils.spring.rmibernate.client.aspects;

import it.openutils.spring.rmibernate.client.pagination.Paginator;
import it.openutils.spring.rmibernate.shared.managers.HibernateLazyService;

import java.io.Serializable;

import net.sf.cglib.proxy.LazyLoader;


/**
 * Aspect that does remote lazy loading on cglib proxy
 * @author mmolaschi
 * @version $Id: HibernateLazyLoaderAspect.java 481 2007-10-25 16:50:39Z fcarone $
 */
public class HibernateLazyLoaderAspect implements LazyLoader, Serializable
{

    /**
     * UID
     */
    private static final long serialVersionUID = -365731708075101363L;

    private static ThreadLocal<Paginator> paginator = new ThreadLocal<Paginator>();

    private String className;

    private String fieldName;

    private Serializable id;

    private Object localReference;

    private HibernateLazyService hibernateLazyService;

    /**
     * Constructor
     * @param className parent class name
     * @param fieldName field to be lazy loaded
     * @param id id of current entity
     */
    public HibernateLazyLoaderAspect(String className, String fieldName, Serializable id, HibernateLazyService hibernateLazyService)
    {
        this.className = className;
        this.fieldName = fieldName;
        this.id = id;
        this.hibernateLazyService = hibernateLazyService;
    }

    /**
     * Set pagination for next calls in current thread
     * @param from starting row number
     * @param size max number of rows
     */
    public static void setPagination(int from, int size)
    {
        paginator.set(new Paginator(from, size));
    }

    /**
     * Disable pagination for next calls in current thread
     */
    public static void resetPagination()
    {
        paginator.set(null);
    }

    /**
     * {@inheritDoc}
     */
    public Object loadObject() throws Exception
    {
        // @todo handle localreference with paginator
        // Load from remote
        if (paginator.get() != null)
        {
            return hibernateLazyService.invoke(className, fieldName, id, paginator.get().getFrom(), paginator.get().getSize());
        }
        if (localReference == null)
        {
            localReference = hibernateLazyService.invoke(className, fieldName, id);
        }
        return localReference;
    }

}
