package it.openutils.spring.rmibernate.shared.managers;

import java.io.Serializable;
import java.rmi.RemoteException;


/**
 * Interface for server-side remote lazy loading calls
 * @author mmolaschi
 * @version $Id: HibernateLazyService.java 481 2007-10-25 16:50:39Z fcarone $
 */
public interface HibernateLazyService
{
    /**
     * Invoke remote lazy loading
     * @param className name of parent class
     * @param propertyName name of property to load
     * @param id current entity id
     * @return loaded object / collection
     * @throws RemoteException error
     */
    Object invoke(String className, String propertyName, Serializable id) throws RemoteException;

    /**
     * Invoke remote lazy loading
     * @param className name of parent class
     * @param propertyName name of property to load
     * @param id current entity id
     * @param from starting row number
     * @param size max rows to return
     * @return loaded object / collection
     * @throws RemoteException error
     */
    Object invoke(String className, String propertyName, Serializable id, int from, int size) throws RemoteException;

    /**
     * Get collection size
     * @param className name of parent class
     * @param propertyName name of property to load
     * @param id current entity id
     * @return loaded object / collection
     * @throws RemoteException error
     */
    long count(String className, String propertyName, Serializable id) throws RemoteException;
}
